﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Threading;
using Arction.Wpf.SemibindableCharting;
using Arction.Wpf.SemibindableCharting.Axes;
using Arction.Wpf.SemibindableCharting.SeriesXY;
using Arction.Wpf.SemibindableCharting.Views.ViewXY;

namespace HeatmapTest
{
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName]string propertyName = null)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        private IntensityPoint[,] intensityPoints = null;
        public IntensityPoint[,] IntensityPoints
        {
            get { return intensityPoints; }
            set
            {
                intensityPoints = value;
                NotifyPropertyChanged();
            }
        }

        public Action UpdateChartInView { get; internal set; }

        DispatcherTimer m_timer;

        const int DataColumnCount = 2000;
        const int DataRowCount = 2000;
        double[][] m_data = null;
        int m_iRound = 0;
        private DateTime startTime;

        public MainWindowViewModel()
        {
            m_timer = new DispatcherTimer();
            m_timer.Interval = TimeSpan.FromMilliseconds(1);
            m_timer.Tick += new EventHandler(m_timer_Tick);
        }

        //Call from View
        internal void StartInVM()
        {
            m_timer.Start();
            startTime = DateTime.Now;
        }

        void m_timer_Tick(object sender, EventArgs e)
        {
            if (m_data == null)
            {
                //Create a data array
                m_data = new double[DataColumnCount][];
                for (int iCol = 0; iCol < DataColumnCount; iCol++)
                {
                    m_data[iCol] = new double[DataRowCount];
                }
                IntensityPoints = ConvertIntesityPoint(m_data);
            }

            //Set new values to data array
            for (int iCol = 0; iCol < DataColumnCount; iCol++)
            {
                for (int iRow = 0; iRow < DataRowCount; iRow++)
                {
                    IntensityPoints[iCol, iRow].Value = (double)m_iRound + 20.0 * Math.Sin((double)iCol * (double)iRow / 5000.0);
                }
            }

            //Call View Method
            UpdateChartInView();

            m_iRound++;
            if (m_iRound == 50)
            {
                Debug.WriteLine($"info Round = {m_iRound}, time = {DateTime.Now - startTime}");
                m_iRound = 0;
            }
        }

        private IntensityPoint[,] ConvertIntesityPoint(double[][] m_data)
        {
            var result = new IntensityPoint[DataColumnCount, DataRowCount];
            for (int iCol = 0; iCol < DataColumnCount; iCol++)
            {
                for (int iRow = 0; iRow < DataRowCount; iRow++)
                {
                    result[iCol, iRow] = new IntensityPoint { Value = m_data[iCol][iRow] };
                }
            }
            return result;
        }
    }
}
